/*
 * This file is part of Sponge, licensed under the MIT License (MIT).
 *
 * Copyright (c) SpongePowered <https://www.spongepowered.org>
 * Copyright (c) contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.spongepowered.common.item.recipe.crafting;

import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.Ingredient;
import org.spongepowered.common.item.inventory.util.ItemStackUtil;

import javax.annotation.Nullable;

/**
 * Delegates a custom implemented {@link org.spongepowered.api.item.recipe.crafting.Ingredient}
 */
public class DelegateIngredient extends Ingredient {

    private org.spongepowered.api.item.recipe.crafting.Ingredient delegate;

    private DelegateIngredient(org.spongepowered.api.item.recipe.crafting.Ingredient delegate) {
        super(ItemStackUtil.fromSnapshotToNative(delegate.displayedItems()));
        this.delegate = delegate;
    }

    @Override
    public boolean apply(@Nullable ItemStack item) {
        return this.delegate.test(ItemStackUtil.fromNative(item));
    }

    public static Ingredient of(org.spongepowered.api.item.recipe.crafting.Ingredient delegate) {
        if ((Object) delegate instanceof Ingredient) {
            return ((Ingredient) (Object) delegate);
        }
        return new DelegateIngredient(delegate);
    }
}
